@@ -23,8 +23,9 @@
 #include "terminal-intl.h"
 #include <glade/glade.h>
 #include <libgnomeui/gnome-color-picker.h>
+#include <libgnomeui/gnome-file-entry.h>
 #include <string.h>
-#include "eggcellrendererkeys.h"
+#include "x-font-selector.h"
 
 /* Bytes in a line of scrollback, rough estimate, including
  * data structure to hold the line. Based on reading
@@ -53,6 +54,20 @@ static TerminalColorScheme color_schemes[] = {
     { 0, 0x0000, 0xFFFF, 0x0000 }, { 0, 0x0000, 0x0000, 0x0000 } }  
 };
 
+typedef struct _TerminalPaletteScheme TerminalPaletteScheme;
+
+struct _TerminalPaletteScheme
+{
+  const char *name;
+  const GdkColor *palette;
+};
+
+static TerminalPaletteScheme palette_schemes[] = {
+  { N_("Linux console"), terminal_palette_linux },
+  { N_("XTerm"), terminal_palette_xterm },
+  { N_("Rxvt"), terminal_palette_rxvt }
+};
+
 static GtkWidget* profile_editor_get_widget                  (GtkWidget       *editor,
                                                               const char      *widget_name);
 static void       profile_editor_update_sensitivity          (GtkWidget       *editor,
@@ -95,6 +110,23 @@ static void       profile_editor_update_use_custom_command   (GtkWidget       *w
                                                               TerminalProfile *profile);
 static void       profile_editor_update_custom_command       (GtkWidget       *widget,
                                                               TerminalProfile *profile);
+static void       profile_editor_update_palette              (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_x_font               (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_background_type      (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_background_image     (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_scroll_background    (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_background_darkness  (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_backspace_binding    (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_delete_binding       (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+
 
 
 static void profile_forgotten (TerminalProfile     *profile,
@@ -117,6 +149,25 @@ entry_set_text_if_changed (GtkEntry   *entry,
   g_free (s);
 }
 
+static void
+colorpicker_set_if_changed (GtkWidget      *colorpicker,
+                            const GdkColor *color)
+{
+  guint16 r, g, b;
+
+  gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (colorpicker),
+                              &r, &g, &b, NULL);
+
+  if (r != color->red ||
+      g != color->green ||
+      b != color->blue)
+    gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (colorpicker),
+                                color->red,
+                                color->green,
+                                color->blue,
+                                0xffff);
+}
+
 static void
 profile_editor_destroyed (GtkWidget       *editor,
                           TerminalProfile *profile)
@@ -209,6 +260,30 @@ profile_changed (TerminalProfile          *profile,
 
   if (mask & TERMINAL_SETTING_CUSTOM_COMMAND)
     profile_editor_update_custom_command (editor, profile);
+
+  if (mask & TERMINAL_SETTING_PALETTE)
+    profile_editor_update_palette (editor, profile);
+
+  if (mask & TERMINAL_SETTING_X_FONT)
+    profile_editor_update_x_font (editor, profile);
+
+  if (mask & TERMINAL_SETTING_BACKGROUND_TYPE)
+    profile_editor_update_background_type (editor, profile);
+  
+  if (mask & TERMINAL_SETTING_BACKGROUND_IMAGE)
+    profile_editor_update_background_image (editor, profile);
+
+  if (mask & TERMINAL_SETTING_SCROLL_BACKGROUND)
+    profile_editor_update_scroll_background (editor, profile);
+
+  if (mask & TERMINAL_SETTING_BACKGROUND_DARKNESS)
+    profile_editor_update_background_darkness (editor, profile);
+
+  if (mask & TERMINAL_SETTING_BACKSPACE_BINDING)
+    profile_editor_update_backspace_binding (editor, profile);
+
+  if (mask & TERMINAL_SETTING_DELETE_BINDING)
+    profile_editor_update_delete_binding (editor, profile);
   
   profile_editor_update_sensitivity (editor, profile);
 }
@@ -449,6 +524,185 @@ custom_command_changed (GtkWidget       *entry,
   g_free (text);
 }
 
+static void
+palette_scheme_changed (GtkWidget       *option_menu,
+                      TerminalProfile *profile)
+{
+  int i;
+  
+  i = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+  
+  if (i < G_N_ELEMENTS (palette_schemes))
+    terminal_profile_set_palette (profile,
+                                  palette_schemes[i].palette);
+  else
+    ; /* "custom" selected, no change */
+}
+
+static void
+palette_color_set (GtkWidget       *colorpicker,
+                   guint r, guint g, guint b, guint a,
+                   TerminalProfile *profile)
+{
+  int i;
+  GdkColor color;
+
+  color.red = r;
+  color.green = g;
+  color.blue = b;
+  
+  i = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (colorpicker),
+                                          "palette-entry-index"));
+
+  terminal_profile_set_palette_entry (profile, i,
+                                      &color);
+}
+
+static void
+font_dialog_destroyed (GtkWidget *dialog,
+                       gpointer   data)
+{
+  GtkWidget *button;
+
+  button = data;
+
+  g_object_set_data (G_OBJECT (button), "font-selector", NULL);
+}
+
+static void
+x_font_clicked (GtkWidget       *button,
+                TerminalProfile *profile)
+{
+  /* Pop up dialog, get new font, terminal_profile_set_x_font */
+  GtkWidget *dialog;
+  GtkWidget *parent;
+  
+  dialog = g_object_get_data (G_OBJECT (button),
+                              "font-selector");
+
+  if (dialog)
+    {
+      gtk_window_present (GTK_WINDOW (dialog));
+      return;
+    }
+  
+  parent = gtk_widget_get_ancestor (button, GTK_TYPE_WINDOW);
+  g_assert (parent);
+
+#if 0
+  dialog = gnome_font_selector_new (_("Choose a font"));
+  
+  gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
+#else
+  dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+                                   GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_INFO,
+                                   GTK_BUTTONS_CLOSE,
+                                   "Sadly, this button doesn't work. "
+                                   "To fix, repair x-font-selector.[hc] "
+                                   "then connect it up in %s (line %d, function %s); all "
+                                   "other parts of the font pref are already done.\n"
+                                   "http://bugzilla.gnome.org/show_bug.cgi?id=71744",
+                                   __FILE__, __LINE__,
+                                   G_GNUC_FUNCTION);
+  g_signal_connect (G_OBJECT (dialog), "response",
+                    G_CALLBACK (gtk_widget_destroy),
+                    NULL);
+#endif
+
+  g_object_set_data (G_OBJECT (button), "font-selector", dialog);
+  g_signal_connect (G_OBJECT (dialog), "destroy",
+                    G_CALLBACK (font_dialog_destroyed), button);
+
+  gtk_window_present (GTK_WINDOW (dialog));
+}
+
+static void
+solid_radio_toggled (GtkWidget       *radiobutton,
+                     TerminalProfile *profile)
+{
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton)))
+    terminal_profile_set_background_type (profile,
+                                          TERMINAL_BACKGROUND_SOLID);
+}
+
+static void
+image_radio_toggled (GtkWidget       *radiobutton,
+                     TerminalProfile *profile)
+{
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton)))
+    terminal_profile_set_background_type (profile,
+                                          TERMINAL_BACKGROUND_IMAGE);
+}
+
+static void
+transparent_radio_toggled (GtkWidget       *radiobutton,
+                           TerminalProfile *profile)
+{
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton)))
+    terminal_profile_set_background_type (profile,
+                                          TERMINAL_BACKGROUND_TRANSPARENT);
+}
+
+static void
+background_image_changed (GtkWidget       *entry,
+                          TerminalProfile *profile)
+{
+  char *text;
+
+  /* We don't use gnome_file_entry_get_full_path() since that would weirdly
+   * insert a directory in front of what the user was typing once the
+   * notify came back from gconf.
+   */
+  
+  text = gtk_editable_get_chars (GTK_EDITABLE (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (entry))),
+                                 0, -1);
+  
+  terminal_profile_set_background_image_file (profile, text);
+
+  g_free (text);
+}
+
+static void
+scroll_background_toggled (GtkWidget       *checkbutton,
+                           TerminalProfile *profile)
+{
+  terminal_profile_set_scroll_background (profile,
+                                          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)));
+}
+
+static void
+darken_background_toggled (GtkWidget       *checkbutton,
+                           TerminalProfile *profile)
+{
+  terminal_profile_set_background_darkness (profile,
+                                            gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)) ?
+                                            DARKNESS_TRUE : DARKNESS_FALSE);
+}
+
+static void
+backspace_binding_changed (GtkWidget       *option_menu,
+                           TerminalProfile *profile)
+{
+  int i;
+  
+  i = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+
+  terminal_profile_set_backspace_binding (profile, i);
+}
+
+static void
+delete_binding_changed (GtkWidget       *option_menu,
+                        TerminalProfile *profile)
+{
+  int i;
+  
+  i = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+
+  terminal_profile_set_delete_binding (profile, i);
+}
+
 /*
  * initialize widgets
  */
@@ -466,7 +720,8 @@ init_color_scheme_menu (GtkWidget *option_menu)
   while (i < G_N_ELEMENTS (color_schemes))
     {
       menu_item = gtk_menu_item_new_with_label (_(color_schemes[i].name));
-
+      gtk_widget_show (menu_item);
+      
       gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                              menu_item);
       
@@ -474,13 +729,44 @@ init_color_scheme_menu (GtkWidget *option_menu)
     }
 
   menu_item = gtk_menu_item_new_with_label (_("Custom"));
+  gtk_widget_show (menu_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          menu_item);
 
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu),
                             menu);
 }
-                        
+
+static void
+init_palette_scheme_menu (GtkWidget *option_menu)
+{
+  GtkWidget *menu;
+  GtkWidget *menu_item;
+  int i;
+  
+  menu = gtk_menu_new ();
+
+  i = 0;
+  while (i < G_N_ELEMENTS (palette_schemes))
+    {
+      menu_item = gtk_menu_item_new_with_label (_(palette_schemes[i].name));
+      gtk_widget_show (menu_item);
+      
+      gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                             menu_item);
+      
+      ++i;
+    }
+
+  menu_item = gtk_menu_item_new_with_label (_("Custom"));
+  gtk_widget_show (menu_item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         menu_item);
+
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu),
+                            menu);
+}
+
 static void
 edited_cb (GtkCellRenderer *renderer,
 	   gchar           *path_text,
@@ -492,7 +778,7 @@ edited_cb (GtkCellRenderer *renderer,
 
   path = gtk_tree_path_new_from_string (path_text);
   gtk_tree_model_get_iter (model, &iter, path);
-  gtk_list_store_set (model, &iter,
+  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 		      0, new_text,
 		      -1);
   gtk_tree_path_free (path);
@@ -513,10 +799,6 @@ terminal_profile_edit (TerminalProfile *profile,
       GladeXML *xml;
       GtkWidget *w;
       double num1, num2;
-      GtkWidget *view;
-      GtkCellRenderer *renderer;
-      GtkTreeModel *model;
-      GtkTreeIter iter;
       gint i;
       
       if (g_file_test ("./"TERM_GLADE_FILE,
@@ -643,7 +925,7 @@ terminal_profile_edit (TerminalProfile *profile,
 
       w = glade_xml_get_widget (xml, "color-scheme-optionmenu");
       init_color_scheme_menu (w);
-      profile_editor_update_color_scheme_menu (editor, profile);
+      profile_editor_update_palette (editor, profile);
       g_signal_connect (G_OBJECT (w), "changed",
                         G_CALLBACK (color_scheme_changed),
                         profile);
@@ -746,28 +1028,93 @@ terminal_profile_edit (TerminalProfile *profile,
                         G_CALLBACK (custom_command_changed),
                         profile);
 
-      w = glade_xml_get_widget (xml, "bindings-swindow");
+      w = glade_xml_get_widget (xml, "palette-optionmenu");
+      g_assert (w);
+      init_palette_scheme_menu (w);
+      g_signal_connect (G_OBJECT (w), "changed",
+                        G_CALLBACK (palette_scheme_changed),
+                        profile);
 
-      model = gtk_list_store_new (1, G_TYPE_STRING);
-      for (i = 0; i < 30; i++)
-	{
-	  gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-	  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			      0, "<control>-l",
-			      -1);
-	}
-      view = gtk_tree_view_new_with_model (model);
-      renderer = g_object_new (EGG_TYPE_CELL_RENDERER_KEYS,
-			       "editable", TRUE,
-			       NULL);
-      g_signal_connect (G_OBJECT (renderer), "edited", edited_cb, model);
-      gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
-						   0,
-						   "Egg Test",
-						   renderer,
-						   "text", 0,
-						   NULL);
-      gtk_container_add (GTK_CONTAINER (w), view);
+      profile_editor_update_background_type (editor, profile);
+      w = glade_xml_get_widget (xml, "solid-radiobutton");
+      g_signal_connect (G_OBJECT (w), "toggled",
+                        G_CALLBACK (solid_radio_toggled),
+                        profile);
+      w = glade_xml_get_widget (xml, "image-radiobutton");
+      g_signal_connect (G_OBJECT (w), "toggled",
+                        G_CALLBACK (image_radio_toggled),
+                        profile);
+      w = glade_xml_get_widget (xml, "transparent-radiobutton");
+      g_signal_connect (G_OBJECT (w), "toggled",
+                        G_CALLBACK (transparent_radio_toggled),
+                        profile);
+
+      w = glade_xml_get_widget (xml, "background-image-fileentry");
+      profile_editor_update_background_image (editor, profile);
+      g_signal_connect (G_OBJECT (w), "changed",
+                        G_CALLBACK (background_image_changed),
+                        profile);
+
+      w = glade_xml_get_widget (xml, "scroll-background-checkbutton");
+      profile_editor_update_scroll_background (editor, profile);
+      g_signal_connect (G_OBJECT (w), "toggled",
+                        G_CALLBACK (scroll_background_toggled),
+                        profile);
+      
+      w = glade_xml_get_widget (xml, "darken-background-checkbutton");
+      profile_editor_update_background_darkness (editor, profile);
+      g_signal_connect (G_OBJECT (w), "toggled",
+                        G_CALLBACK (darken_background_toggled),
+                        profile);
+
+      w = glade_xml_get_widget (xml, "backspace-binding-optionmenu");
+      profile_editor_update_backspace_binding (editor, profile);
+      g_signal_connect (G_OBJECT (w), "changed",
+                        G_CALLBACK (backspace_binding_changed),
+                        profile);
+
+      w = glade_xml_get_widget (xml, "delete-binding-optionmenu");
+      profile_editor_update_delete_binding (editor, profile);
+      g_signal_connect (G_OBJECT (w), "changed",
+                        G_CALLBACK (delete_binding_changed),
+                        profile);
+      
+      i = 0;
+      while (i < TERMINAL_PALETTE_SIZE)
+        {
+          char *s = g_strdup_printf ("palette-colorpicker-%d", i+1);
+          
+          w = glade_xml_get_widget (xml, s);
+          g_assert (w);
+          
+          g_object_set_data (G_OBJECT (w),
+                             "palette-entry-index",
+                             GINT_TO_POINTER (i));
+
+          g_signal_connect (G_OBJECT (w), "color_set",
+                            G_CALLBACK (palette_color_set),
+                            profile);
+          
+          g_free (s);
+
+          ++i;
+        }
+
+      profile_editor_update_palette (editor, profile);
+
+      w = glade_xml_get_widget (xml, "font-button");
+      /* keep button from expanding the dialog, and truncate on
+       * the left not on either side; would really prefer to
+       * center until running out of space, but oh well.
+       */
+      gtk_misc_set_alignment (GTK_MISC (gtk_bin_get_child (GTK_BIN (w))),
+                              0.0, 0.5);
+      gtk_widget_set_size_request (gtk_bin_get_child (GTK_BIN (w)),
+                                   0, -1);
+      profile_editor_update_x_font (editor, profile);
+      g_signal_connect (G_OBJECT (w), "clicked",
+                        G_CALLBACK (x_font_clicked),
+                        profile);
     }
   else
     {
@@ -781,7 +1128,7 @@ terminal_profile_edit (TerminalProfile *profile,
       gtk_widget_hide (editor); /* re-show the window on its new parent */
     }
   
-  gtk_widget_show_all (editor);
+  /*   gtk_widget_show_all (editor);*/
   gtk_window_present (GTK_WINDOW (editor));
 }
 
@@ -820,7 +1167,21 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
   w = profile_editor_get_widget (editor, "custom-command-entry");
   gtk_widget_set_sensitive (w,
                             !((mask & TERMINAL_SETTING_CUSTOM_COMMAND) ||
-                              !terminal_profile_get_use_custom_command (profile)));    
+                              !terminal_profile_get_use_custom_command (profile))); 
+  if (terminal_profile_get_background_type (profile) == TERMINAL_BACKGROUND_IMAGE)
+    {
+      w = profile_editor_get_widget (editor, "background-image-fileentry");
+      gtk_widget_set_sensitive (w, !(mask & TERMINAL_SETTING_BACKGROUND_IMAGE));
+      w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
+      gtk_widget_set_sensitive (w, !(mask & TERMINAL_SETTING_SCROLL_BACKGROUND));
+    }
+  else
+    {
+      w = profile_editor_get_widget (editor, "background-image-fileentry");
+      gtk_widget_set_sensitive (w, FALSE);
+      w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
+      gtk_widget_set_sensitive (w, FALSE);
+    }
   
 #if 0 /* uncomment once we've tested the sensitivity code */
   if (mask == last_mask)
@@ -890,6 +1251,43 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
 
   set_insensitive (editor, "use-custom-command-checkbutton",
                    mask & TERMINAL_SETTING_USE_CUSTOM_COMMAND);
+
+  set_insensitive (editor, "palette-optionmenu",
+                   mask & TERMINAL_SETTING_PALETTE);
+
+  set_insensitive (editor, "font-button",
+                   mask & TERMINAL_SETTING_X_FONT);
+
+  set_insensitive (editor, "solid-radiobutton",
+                   mask & TERMINAL_SETTING_BACKGROUND_TYPE);
+  set_insensitive (editor, "image-radiobutton",
+                   mask & TERMINAL_SETTING_BACKGROUND_TYPE);
+  set_insensitive (editor, "transparent-radiobutton",
+                   mask & TERMINAL_SETTING_BACKGROUND_TYPE);
+
+  set_insensitive (editor, "darken-background-checkbutton",
+                   mask & TERMINAL_SETTING_BACKGROUND_DARKNESS);
+
+  set_insensitive (editor, "backspace-binding-optionmenu",
+                   mask & TERMINAL_SETTING_BACKSPACE_BINDING);
+  set_insensitive (editor, "delete-binding-optionmenu",
+                   mask & TERMINAL_SETTING_DELETE_BINDING);
+  
+  {
+    int i;
+
+    i = 0;
+    while (i < TERMINAL_PALETTE_SIZE)
+      {
+        char *s = g_strdup_printf ("palette-colorpicker-%d", i+1);
+
+        set_insensitive (editor, s, mask & TERMINAL_SETTING_PALETTE);
+
+        g_free (s);
+
+        ++i;
+      }
+  }
 }
 
 
@@ -920,8 +1318,8 @@ profile_editor_update_cursor_blink (GtkWidget       *editor,
   GtkWidget *w;
 
   w = profile_editor_get_widget (editor, "blink-cursor-checkbutton");
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w),
-                               terminal_profile_get_cursor_blink (profile));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_cursor_blink (profile));
 }
 
 
@@ -932,8 +1330,8 @@ profile_editor_update_default_show_menubar (GtkWidget       *editor,
   GtkWidget *w;
 
   w = profile_editor_get_widget (editor, "show-menubar-checkbutton");
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w),
-                               terminal_profile_get_default_show_menubar (profile));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_default_show_menubar (profile));
 }
 
 static void
@@ -946,12 +1344,10 @@ profile_editor_update_color_pickers (GtkWidget       *editor,
   terminal_profile_get_color_scheme (profile, &color1, &color2);
   
   w = profile_editor_get_widget (editor, "foreground-colorpicker");
-  gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (w),
-                              color1.red, color1.green, color1.blue, 0);
+  colorpicker_set_if_changed (w, &color1);
   
   w = profile_editor_get_widget (editor, "background-colorpicker");
-  gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (w),
-                              color2.red, color2.green, color2.blue, 0);
+  colorpicker_set_if_changed (w, &color2);
 }
 
 static void
@@ -1015,8 +1411,8 @@ profile_editor_update_allow_bold (GtkWidget       *editor,
 
   w = profile_editor_get_widget (editor, "allow-bold-checkbutton");
   
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w),
-                               terminal_profile_get_allow_bold (profile));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_allow_bold (profile));
 }
 
 static void
@@ -1027,8 +1423,8 @@ profile_editor_update_silent_bell (GtkWidget       *editor,
 
   w = profile_editor_get_widget (editor, "silent-bell-checkbutton");
   
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w),
-                               terminal_profile_get_silent_bell (profile));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_silent_bell (profile));
 }
 
 static void
@@ -1081,8 +1477,8 @@ profile_editor_update_scroll_on_keystroke  (GtkWidget       *editor,
 
   w = profile_editor_get_widget (editor, "scroll-on-keystroke-checkbutton");
   
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w),
-                               terminal_profile_get_scroll_on_keystroke (profile));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_scroll_on_keystroke (profile));
 }
 
 static void
@@ -1093,8 +1489,8 @@ profile_editor_update_scroll_on_output (GtkWidget       *editor,
 
   w = profile_editor_get_widget (editor, "scroll-on-output-checkbutton");
   
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w),
-                               terminal_profile_get_scroll_on_output (profile));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_scroll_on_output (profile));
 }
 
 static void
@@ -1117,8 +1513,8 @@ profile_editor_update_login_shell (GtkWidget       *editor,
 
   w = profile_editor_get_widget (editor, "login-shell-checkbutton");
   
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w),
-                               terminal_profile_get_login_shell (profile));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_login_shell (profile));
 }
 
 static void
@@ -1129,7 +1525,7 @@ profile_editor_update_update_records (GtkWidget       *editor,
 
   w = profile_editor_get_widget (editor, "update-records-checkbutton");
   
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w),
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
                                terminal_profile_get_update_records (profile));
 
 }
@@ -1142,7 +1538,7 @@ profile_editor_update_use_custom_command (GtkWidget       *editor,
 
   w = profile_editor_get_widget (editor, "use-custom-command-checkbutton");
   
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w),
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
                                terminal_profile_get_use_custom_command (profile));
 
 }
@@ -1182,6 +1578,163 @@ profile_editor_update_custom_command (GtkWidget       *editor,
     }            
 }
 
+static void
+profile_editor_update_palette (GtkWidget       *editor,
+                               TerminalProfile *profile)
+{
+  GtkWidget *w;
+  int i;
+  GdkColor palette[TERMINAL_PALETTE_SIZE];
+
+  terminal_profile_get_palette (profile, palette);
+  
+  i = 0;
+  while (i < TERMINAL_PALETTE_SIZE)
+    {
+      char *s = g_strdup_printf ("palette-colorpicker-%d", i+1);
+      w = profile_editor_get_widget (editor, s);
+      g_free (s);
+      
+      colorpicker_set_if_changed (w, &palette[i]);
+      
+      ++i;
+    }
+
+  w = profile_editor_get_widget (editor, "palette-optionmenu");
+
+  i = 0;
+  while (i < G_N_ELEMENTS (palette_schemes))
+    {
+      int j;
+      gboolean match;
+
+      match = TRUE;
+      j = 0;
+      while (j < TERMINAL_PALETTE_SIZE)
+        {
+          if (!gdk_color_equal (&palette_schemes[i].palette[j],
+                                &palette[j]))
+            {
+              match = FALSE;
+              break;
+            }
+          
+          ++j;
+        }
+
+      if (match)
+        break;
+      
+      ++i;
+    }
+
+  /* If we didn't find a match, then we want the last option
+   * menu item which is "custom"
+   */
+  gtk_option_menu_set_history (GTK_OPTION_MENU (w), i);
+}
+
+static void
+profile_editor_update_x_font (GtkWidget       *editor,
+                              TerminalProfile *profile)
+{
+  GtkWidget *w;
+  GtkWidget *child;
+  
+  w = profile_editor_get_widget (editor, "font-button");  
+  
+  child = gtk_bin_get_child (GTK_BIN (w));
+
+  gtk_label_set_text (GTK_LABEL (child),
+                      terminal_profile_get_x_font (profile));
+}
+
+static void
+profile_editor_update_background_type (GtkWidget       *editor,
+                                       TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+  switch (terminal_profile_get_background_type (profile))
+    {
+    case TERMINAL_BACKGROUND_SOLID:
+      w = profile_editor_get_widget (editor, "solid-radiobutton");
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
+      break;
+    case TERMINAL_BACKGROUND_IMAGE:
+      w = profile_editor_get_widget (editor, "image-radiobutton");
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
+      break;
+    case TERMINAL_BACKGROUND_TRANSPARENT:
+      w = profile_editor_get_widget (editor, "transparent-radiobutton");
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
+      break;
+    }
+}
+
+static void
+profile_editor_update_background_image (GtkWidget       *editor,
+                                        TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+  w = profile_editor_get_widget (editor, "background-image-fileentry");
+
+  entry_set_text_if_changed (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (w))),
+                             terminal_profile_get_background_image_file (profile));
+}
+
+static void
+profile_editor_update_scroll_background (GtkWidget       *editor,
+                                         TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+  w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                               terminal_profile_get_scroll_background (profile));
+}
+
+static void
+profile_editor_update_background_darkness (GtkWidget       *editor,
+                                           TerminalProfile *profile)
+{
+  GtkWidget *w;
+  double v;
+
+  w = profile_editor_get_widget (editor, "darken-background-checkbutton");
+
+  v = terminal_profile_get_background_darkness (profile);
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                v >= DARKNESS_THRESHOLD);
+}
+
+static void
+profile_editor_update_backspace_binding (GtkWidget       *editor,
+                                         TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+  w = profile_editor_get_widget (editor, "backspace-binding-optionmenu");
+  
+  gtk_option_menu_set_history (GTK_OPTION_MENU (w),
+                               terminal_profile_get_backspace_binding (profile));
+}
+
+static void
+profile_editor_update_delete_binding (GtkWidget       *editor,
+                                      TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+  w = profile_editor_get_widget (editor, "delete-binding-optionmenu");
+  
+  gtk_option_menu_set_history (GTK_OPTION_MENU (w),
+                               terminal_profile_get_delete_binding (profile));
+}
+
 static GtkWidget*
 profile_editor_get_widget (GtkWidget  *editor,
                            const char *widget_name)
@@ -1193,11 +1746,11 @@ profile_editor_get_widget (GtkWidget  *editor,
                            "glade-xml");
 
   g_return_val_if_fail (xml, NULL);
-
+  
   w = glade_xml_get_widget (xml, widget_name);
 
   if (w == NULL)
-    g_warning ("No such widget %s", widget_name);
+    g_error ("No such widget %s", widget_name);
   
   return w;
 }
