@@ -28,7 +28,8 @@
 #define CONF_PREFIX "/apps/gnome-terminal"
 #define CONF_GLOBAL_PREFIX CONF_PREFIX"/global"
 #define CONF_PROFILES_PREFIX CONF_PREFIX"/profiles"
-#define DEFAULT_PROFILE "Default"
+#define CONF_KEYS_PREFIX CONF_PREFIX"/keybindings"
+#define FALLBACK_PROFILE_ID "Default"
 
 typedef enum
 {
@@ -51,7 +52,18 @@ typedef enum
   TERMINAL_SETTING_UPDATE_RECORDS       = 1 << 16,
   TERMINAL_SETTING_USE_CUSTOM_COMMAND   = 1 << 17,
   TERMINAL_SETTING_CUSTOM_COMMAND       = 1 << 18,
-  TERMINAL_SETTING_ICON                 = 1 << 19
+  TERMINAL_SETTING_ICON                 = 1 << 19,
+  TERMINAL_SETTING_IS_DEFAULT           = 1 << 20,
+  TERMINAL_SETTING_PALETTE              = 1 << 21,
+  TERMINAL_SETTING_X_FONT               = 1 << 22,
+  TERMINAL_SETTING_BACKGROUND_TYPE      = 1 << 23,
+  TERMINAL_SETTING_BACKGROUND_IMAGE     = 1 << 24,
+  TERMINAL_SETTING_SCROLL_BACKGROUND    = 1 << 25,
+  TERMINAL_SETTING_BACKGROUND_DARKNESS  = 1 << 26,
+  TERMINAL_SETTING_BACKSPACE_BINDING    = 1 << 27,
+  TERMINAL_SETTING_DELETE_BINDING       = 1 << 28
+
+  /* When we get to 31 we need to do something else ;-) */
 } TerminalSettingMask;
 
 typedef enum
@@ -67,18 +79,12 @@ typedef enum
 
 typedef enum
 {
-  TERMINAL_DELETE_CONTROL_H,
-  TERMINAL_DELETE_ESCAPE_SEQUENCE,
-  TERMINAL_DELETE_ASCII_DEL
-} TerminalDeleteBinding;
+  TERMINAL_ERASE_ASCII_DEL,
+  TERMINAL_ERASE_ESCAPE_SEQUENCE,
+  TERMINAL_ERASE_CONTROL_H
+} TerminalEraseBinding;
 
-typedef enum
-{
-  TERMINAL_PALETTE_LINUX,
-  TERMINAL_PALETTE_XTERM,
-  TERMINAL_PALETTE_RXVT,
-  TERMINAL_PALETTE_CUSTOM
-} TerminalPaletteType;
+#define TERMINAL_PALETTE_SIZE 16
 
 typedef enum
 {
@@ -93,6 +99,13 @@ typedef enum
   TERMINAL_EXIT_RESTART
 } TerminalExitAction;
 
+typedef enum
+{
+  TERMINAL_BACKGROUND_SOLID,
+  TERMINAL_BACKGROUND_IMAGE,
+  TERMINAL_BACKGROUND_TRANSPARENT
+} TerminalBackgroundType;
+
 G_BEGIN_DECLS
 
 #define TERMINAL_TYPE_PROFILE              (terminal_profile_get_type ())
@@ -153,6 +166,19 @@ const char*               terminal_profile_get_custom_command       (TerminalPro
 
 const char*               terminal_profile_get_icon_file            (TerminalProfile *profile);
 GdkPixbuf*                terminal_profile_get_icon                 (TerminalProfile *profile);
+gboolean                  terminal_profile_get_is_default           (TerminalProfile *profile);
+void                      terminal_profile_get_palette              (TerminalProfile *profile,
+                                                                     GdkColor        *colors);
+const char*               terminal_profile_get_x_font               (TerminalProfile *profile);
+
+TerminalBackgroundType terminal_profile_get_background_type       (TerminalProfile *profile);
+GdkPixbuf*             terminal_profile_get_background_image      (TerminalProfile *profile);
+const char*            terminal_profile_get_background_image_file (TerminalProfile *profile);
+gboolean               terminal_profile_get_scroll_background     (TerminalProfile *profile);
+double                 terminal_profile_get_background_darkness   (TerminalProfile *profile);
+TerminalEraseBinding   terminal_profile_get_backspace_binding     (TerminalProfile *profile);
+TerminalEraseBinding   terminal_profile_get_delete_binding        (TerminalProfile *profile);
+
 
 void terminal_profile_set_cursor_blink         (TerminalProfile           *profile,
                                                 gboolean                   setting);
@@ -195,12 +221,40 @@ void terminal_profile_set_custom_command       (TerminalProfile          *profil
 
 void terminal_profile_set_icon_file            (TerminalProfile          *profile,
                                                 const char               *filename);
+void terminal_profile_set_is_default           (TerminalProfile          *profile,
+                                                gboolean                  setting);
+void terminal_profile_set_palette              (TerminalProfile *profile,
+                                                const GdkColor  *colors);
+void terminal_profile_set_palette_entry        (TerminalProfile *profile,
+                                                int              i,
+                                                const GdkColor  *color);
+void terminal_profile_set_x_font               (TerminalProfile *profile,
+                                                const char      *name);
 
-void             terminal_profile_setup_default (GConfClient *conf);
-GList*           terminal_profile_get_list (void);
+void terminal_profile_set_background_type       (TerminalProfile        *profile,
+                                                 TerminalBackgroundType  type);
+void terminal_profile_set_background_image_file (TerminalProfile        *profile,
+                                                 const char             *filename);
+void terminal_profile_set_scroll_background     (TerminalProfile        *profile,
+                                                 gboolean                setting);
+void terminal_profile_set_background_darkness   (TerminalProfile        *profile,
+                                                 double                  setting);
+void terminal_profile_set_backspace_binding     (TerminalProfile        *profile,
+                                                 TerminalEraseBinding    binding);
+void terminal_profile_set_delete_binding        (TerminalProfile        *profile,
+                                                 TerminalEraseBinding    binding);
+
+TerminalProfile* terminal_profile_ensure_fallback        (GConfClient     *conf);
+void             terminal_profile_initialize             (GConfClient     *conf);
+GList*           terminal_profile_get_list               (void);
+int              terminal_profile_get_count              (void);
+/* may return NULL */
+TerminalProfile* terminal_profile_get_default            (void);
+/* never returns NULL if any profiles exist, one is always supposed to */
+TerminalProfile* terminal_profile_get_for_new_term       (void);
 TerminalProfile* terminal_profile_lookup                 (const char      *name);
 TerminalProfile* terminal_profile_lookup_by_visible_name (const char      *name);
-void             terminal_profile_forget   (TerminalProfile *profile);
+void             terminal_profile_forget                 (TerminalProfile *profile);
 
 TerminalSettingMask terminal_profile_get_locked_settings (TerminalProfile *profile);
 
@@ -214,6 +268,23 @@ void terminal_profile_delete_list (GConfClient *conf,
                                    GList      *list,
                                    GtkWindow  *transient_parent);
 
+extern const GdkColor terminal_palette_linux[TERMINAL_PALETTE_SIZE];
+extern const GdkColor terminal_palette_xterm[TERMINAL_PALETTE_SIZE];
+extern const GdkColor terminal_palette_rxvt[TERMINAL_PALETTE_SIZE];
+
+char*    terminal_palette_to_string   (const GdkColor *palette);
+gboolean terminal_palette_from_string (const char     *str,
+                                       GdkColor       *palette,
+                                       gboolean        warn);
+
+
+/* Used to convert the double "darkness" setting into a bool.
+ * Temporary hack.
+ */
+#define DARKNESS_THRESHOLD 0.25
+#define DARKNESS_TRUE  0.5
+#define DARKNESS_FALSE 0.0
+
 G_END_DECLS
 
 #endif /* TERMINAL_PROFILE_H */
