@@ -19,12 +19,18 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#undef GDK_DISABLE_DEPRECATED
+#include <gdk/gdkfont.h>
+#define GDK_DISABLE_DEPRECATED
+
 #include "terminal-intl.h"
+#include "terminal-accels.h"
 #include "terminal-window.h"
 #include "terminal-profile.h"
 #include "terminal.h"
 #include <libzvt/libzvt.h>
 #include <libgnome/gnome-util.h> /* gnome_util_user_shell */
+#include <libgnome/gnome-url.h> /* gnome_url_show */
 #include <gdk/gdkx.h>
 #include <string.h>
 #include <stdlib.h>
@@ -43,6 +49,8 @@ struct _TerminalScreenPrivate
   GtkWidget *popup_menu;
   char *raw_title;
   char *cooked_title;
+  char *matched_string;
+  char **override_command;
 };
 
 static GList* used_ids = NULL;
@@ -125,6 +133,13 @@ terminal_screen_init (TerminalScreen *screen)
   gtk_object_sink (GTK_OBJECT (screen->priv->zvt));
 
   zvt_term_set_auto_window_hint (ZVT_TERM (screen->priv->zvt), FALSE);
+
+  zvt_term_match_add (ZVT_TERM (screen->priv->zvt),
+                     "(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?",
+                     VTATTR_UNDERLINE, "host only url");
+  zvt_term_match_add (ZVT_TERM (screen->priv->zvt),
+                      "(((news|telnet|nttp|file|http|ftp|https)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?/[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\) ,\\\"]",
+                      VTATTR_UNDERLINE, "full url");
   
   g_object_set_data (G_OBJECT (screen->priv->zvt),
                      "terminal-screen",
@@ -159,6 +174,8 @@ terminal_screen_init (TerminalScreen *screen)
                     "selection_changed",
                     G_CALLBACK (terminal_screen_zvt_selection_changed),
                     screen);
+
+  gtk_widget_show (screen->priv->zvt);
 }
 
 static void
@@ -213,6 +230,8 @@ terminal_screen_finalize (GObject *object)
 
   g_free (screen->priv->raw_title);
   g_free (screen->priv->cooked_title);
+  g_free (screen->priv->matched_string);
+  g_strfreev (screen->priv->override_command);
   
   g_free (screen->priv);
   
@@ -279,6 +298,7 @@ reread_profile (TerminalScreen *screen)
 {
   TerminalProfile *profile;
   ZvtTerm *term;
+  int bgflags;
   
   profile = screen->priv->profile;  
   
@@ -286,11 +306,27 @@ reread_profile (TerminalScreen *screen)
     return;
 
   term = ZVT_TERM (screen->priv->zvt);
+
+  if (screen->priv->window)
+    {
+      /* We need these in line for the set_size in
+       * update_on_realize
+       */
+      terminal_window_update_scrollbar (screen->priv->window, screen);
+      terminal_window_update_icon (screen->priv->window);
+      terminal_window_update_geometry (screen->priv->window);
+    }
   
   if (GTK_WIDGET_REALIZED (screen->priv->zvt))
     terminal_screen_update_on_realize (term, screen);
 
   rebuild_title (screen);
+
+  /* FIXME For now, just hardwire the backspace and delete keys correctly
+   * Needs to be wired up for people that expect brokenness later.
+   */
+  zvt_term_set_del_key_swap (term, TRUE);
+  zvt_term_set_del_is_del (term, FALSE);
   
   zvt_term_set_blink (term,
                       terminal_profile_get_cursor_blink (profile));
@@ -310,11 +346,45 @@ reread_profile (TerminalScreen *screen)
   zvt_term_set_scrollback (term,
                            terminal_profile_get_scrollback_lines (profile));
 
-  if (screen->priv->window)
+  bgflags = 0;
+  if (terminal_profile_get_scroll_background (profile))
+    bgflags |= ZVT_BACKGROUND_SCROLL;
+  if (terminal_profile_get_background_darkness (profile) > DARKNESS_THRESHOLD)
+    bgflags |= ZVT_BACKGROUND_SHADED;
+  
+  zvt_term_set_background (term,
+                           terminal_profile_get_background_type (profile) ==
+                           TERMINAL_BACKGROUND_IMAGE ?
+                           terminal_profile_get_background_image_file (profile) :
+                           NULL,
+                           terminal_profile_get_background_type (profile) ==
+                           TERMINAL_BACKGROUND_TRANSPARENT,
+                           bgflags);
+
+  switch (terminal_profile_get_backspace_binding (profile))
     {
-      terminal_window_update_scrollbar (screen->priv->window, screen);
-      terminal_window_update_icon (screen->priv->window);
-      terminal_window_update_geometry (screen->priv->window);
+    case TERMINAL_ERASE_CONTROL_H:
+      zvt_term_set_backspace_binding (term, ZVT_ERASE_CONTROL_H);
+      break;
+    case TERMINAL_ERASE_ESCAPE_SEQUENCE:
+      zvt_term_set_backspace_binding (term, ZVT_ERASE_ESCAPE_SEQUENCE);
+      break;
+    case TERMINAL_ERASE_ASCII_DEL:
+      zvt_term_set_backspace_binding (term, ZVT_ERASE_ASCII_DEL);
+      break;
+    }
+
+  switch (terminal_profile_get_delete_binding (profile))
+    {
+    case TERMINAL_ERASE_CONTROL_H:
+      zvt_term_set_delete_binding (term, ZVT_ERASE_CONTROL_H);
+      break;
+    case TERMINAL_ERASE_ESCAPE_SEQUENCE:
+      zvt_term_set_delete_binding (term, ZVT_ERASE_ESCAPE_SEQUENCE);
+      break;
+    case TERMINAL_ERASE_ASCII_DEL:
+      zvt_term_set_delete_binding (term, ZVT_ERASE_ASCII_DEL);
+      break;
     }
 }
 
@@ -371,36 +441,33 @@ profile_changed_callback (TerminalProfile          *profile,
   reread_profile (screen);
 }
 
-
-/* FIXME temporary hack */
-static gushort xterm_red[] = { 0x0000, 0x6767, 0x0000, 0x6767, 0x0000, 0x6767, 0x0000, 0x6868,
-                               0x2a2a, 0xffff, 0x0000, 0xffff, 0x0000, 0xffff, 0x0000, 0xffff,
-                               0x0,    0x0 };
-
-static gushort xterm_green[] = { 0x0000, 0x0000, 0x6767, 0x6767, 0x0000, 0x0000, 0x6767, 0x6868,
-                                 0x2a2a, 0x0000, 0xffff, 0xffff, 0x0000, 0x0000, 0xffff, 0xffff,
-                                 0x0,    0x0 };
-static gushort xterm_blue[] = { 0x0000, 0x0000, 0x0000, 0x0000, 0x6767, 0x6767, 0x6767, 0x6868,
-                                0x2a2a, 0x0000, 0x0000, 0x0000, 0xffff, 0xffff, 0xffff, 0xffff,
-                                0x0,    0x0 };
-
 static void
 update_color_scheme (TerminalScreen *screen)
 {
   GdkColor c;
-  gushort red[18],green[18],blue[18];
+  gushort red[18], green[18], blue[18];
   ZvtTerm *term;
   GdkColor fg, bg;
+  int i;
+  GdkColor palette[TERMINAL_PALETTE_SIZE];
   
   if (screen->priv->zvt == NULL ||
       !GTK_WIDGET_REALIZED (screen->priv->zvt))
     return;
   
   term = ZVT_TERM (screen->priv->zvt);
+
+  terminal_profile_get_palette (screen->priv->profile,
+                                palette);
   
-  memcpy (red, xterm_red, sizeof (xterm_red));
-  memcpy (green, xterm_green, sizeof (xterm_green));
-  memcpy (blue, xterm_blue, sizeof (xterm_blue));
+  i = 0;
+  while (i < 16)
+    {
+      red[i] = palette[i].red;
+      green[i] = palette[i].green;
+      blue[i] = palette[i].blue;
+      ++i;
+    }
 
   terminal_profile_get_color_scheme (screen->priv->profile,
                                      &fg, &bg);
@@ -414,7 +481,7 @@ update_color_scheme (TerminalScreen *screen)
   blue[17] = bg.blue;
   
   zvt_term_set_color_scheme (term, red, green, blue);
-  c = term->colors [17];
+  c = term->colors[17];
 
   gdk_window_set_background (GTK_WIDGET (term)->window, &c);
   gtk_widget_queue_draw (GTK_WIDGET (term));
@@ -425,36 +492,45 @@ terminal_screen_update_on_realize (ZvtTerm        *term,
                                    TerminalScreen *screen)
 {
   TerminalProfile *profile;
-  GdkFont *libzvt_workaround_hack;
+  GdkFont *font;
   
   profile = screen->priv->profile;
   
   update_color_scheme (screen);
-
-  /* I fixed this zvt bug but working around it for a couple weeks.
-   * FIXME take this out.
-   */
-  libzvt_workaround_hack = term->font;
-  gdk_font_ref (libzvt_workaround_hack);
-
-  zvt_term_set_fonts (term, term->font,
+  
+  font = gdk_fontset_load (terminal_profile_get_x_font (profile));
+  if (font == NULL)
+    {
+      font = term->font; /* leave it unchanged */
+      gdk_font_ref (font);
+      
+      g_printerr (_("Could not load font \"%s\"\n"),
+                  terminal_profile_get_x_font (profile));
+    }
+  
+  zvt_term_set_fonts (term, font,
                       terminal_profile_get_allow_bold (profile) ?
-                      term->font : NULL);
+                      NULL : font);
 
-  gdk_font_unref (libzvt_workaround_hack);
+  gdk_font_unref (font);
+  
+  /* FIXME s/FALSE/TRUE/ if the font has changed */
+  terminal_window_set_size (screen->priv->window, screen, FALSE);
 }
 
 static void
 profile_forgotten_callback (TerminalProfile *profile,
                             TerminalScreen  *screen)
 {
-  TerminalProfile *default_profile;
+  TerminalProfile *new_profile;
 
-  /* Revert to the default profile */
-  default_profile = terminal_profile_lookup (DEFAULT_PROFILE);
-  g_assert (default_profile);
+  /* Revert to the new term profile if any */
+  new_profile = terminal_profile_get_for_new_term ();
 
-  terminal_screen_set_profile (screen, default_profile);
+  if (new_profile)
+    terminal_screen_set_profile (screen, new_profile);
+  else
+    g_assert_not_reached (); /* FIXME ? */
 }
 
 void
@@ -519,6 +595,25 @@ terminal_screen_get_profile (TerminalScreen *screen)
   return screen->priv->profile;
 }
 
+void
+terminal_screen_set_override_command (TerminalScreen *screen,
+                                      char          **argv)
+{
+  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+
+  g_strfreev (screen->priv->override_command);
+  screen->priv->override_command = g_strdupv (argv);
+}
+
+const char**
+terminal_screen_get_override_command (TerminalScreen *screen)
+{
+  g_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
+
+  return (const char**) screen->priv->override_command;
+}
+
+
 GtkWidget*
 terminal_screen_get_widget (TerminalScreen *screen)
 {
@@ -534,7 +629,7 @@ show_pty_error_dialog (TerminalScreen *screen,
   dialog = gtk_message_dialog_new ((GtkWindow*)
                                    gtk_widget_get_ancestor (screen->priv->zvt,
                                                             GTK_TYPE_WINDOW),
-                                   0,
+                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_ERROR,
                                    GTK_BUTTONS_CLOSE,
                                    _("There was an error creating the child process for this terminal: %s"),
@@ -559,7 +654,7 @@ show_command_error_dialog (TerminalScreen *screen,
   dialog = gtk_message_dialog_new ((GtkWindow*)
                                    gtk_widget_get_ancestor (screen->priv->zvt,
                                                             GTK_TYPE_WINDOW),
-                                   0,
+                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_ERROR,
                                    GTK_BUTTONS_CLOSE,
                                    _("There was a problem with the command for this terminal: %s"),
@@ -776,8 +871,13 @@ get_child_command (TerminalScreen *screen,
     *file_p = NULL;
   if (argv_p)
     *argv_p = NULL;
-  
-  if (terminal_profile_get_use_custom_command (profile))
+
+  if (screen->priv->override_command)
+    {
+      file = g_strdup (screen->priv->override_command[0]);
+      argv = g_strdupv (screen->priv->override_command);
+    }
+  else if (terminal_profile_get_use_custom_command (profile))
     {
       if (!g_shell_parse_argv (terminal_profile_get_custom_command (profile),
                                NULL, &argv,
@@ -962,25 +1062,6 @@ terminal_screen_get_text_selected (TerminalScreen *screen)
     return FALSE;
 }
 
-static void
-not_implemented (void)
-{
-  GtkWidget *dialog;
-
-  dialog = gtk_message_dialog_new (NULL,
-                                   0,
-                                   GTK_MESSAGE_ERROR,
-                                   GTK_BUTTONS_CLOSE,
-                                   "Didn't implement this item yet, sorry");
-  g_signal_connect (G_OBJECT (dialog), "response",
-                    G_CALLBACK (gtk_widget_destroy),
-                    NULL);
-
-  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-  
-  gtk_widget_show (dialog);
-}
-
 static void
 new_window_callback (GtkWidget      *menu_item,
                      TerminalScreen *screen)
@@ -988,7 +1069,7 @@ new_window_callback (GtkWidget      *menu_item,
   terminal_app_new_terminal (terminal_app_get (),
                              screen->priv->profile,
                              NULL,
-                             FALSE, FALSE);
+                             FALSE, FALSE, NULL, NULL);
 }
 
 static void
@@ -998,7 +1079,21 @@ new_tab_callback (GtkWidget      *menu_item,
   terminal_app_new_terminal (terminal_app_get (),
                              screen->priv->profile,
                              screen->priv->window,
-                             FALSE, FALSE);
+                             FALSE, FALSE, NULL, NULL);
+}
+
+static void
+copy_callback (GtkWidget      *menu_item,
+               TerminalScreen *screen)
+{
+  zvt_term_copy_clipboard (ZVT_TERM (screen->priv->zvt));
+}
+
+static void
+paste_callback (GtkWidget      *menu_item,
+                TerminalScreen *screen)
+{
+  zvt_term_paste_clipboard (ZVT_TERM (screen->priv->zvt));
 }
 
 static void
@@ -1009,7 +1104,7 @@ configuration_callback (GtkWidget      *menu_item,
   
   terminal_app_edit_profile (terminal_app_get (),
                              screen->priv->profile, 
-                             screen->priv->window);
+                             GTK_WINDOW (screen->priv->window));
 }
 
 static void
@@ -1044,12 +1139,88 @@ show_menubar_callback (GtkWidget      *menu_item,
                                          TRUE);
 }
 
+#if 0
 static void
 secure_keyboard_callback (GtkWidget      *menu_item,
                           TerminalScreen *screen)
 {
   not_implemented ();
 }
+#endif
+
+static void
+open_url (TerminalScreen *screen,
+          const char     *orig_url)
+{
+  GError *err;
+  GtkWidget *dialog;
+  char *url;
+  
+  g_return_if_fail (orig_url != NULL);
+
+  /* this is to handle gnome_url_show reentrancy */
+  url = g_strdup (orig_url);
+  g_object_ref (G_OBJECT (screen));
+  
+  err = NULL;
+  gnome_url_show (url, &err);
+
+  if (err)
+    {
+      GtkWidget *window;
+
+      if (screen->priv->zvt)
+        window = gtk_widget_get_ancestor (screen->priv->zvt,
+                                          GTK_TYPE_WINDOW);
+      else
+        window = NULL;
+      
+      dialog = gtk_message_dialog_new (window ? GTK_WINDOW (window) : NULL,
+                                       GTK_DIALOG_DESTROY_WITH_PARENT,
+                                       GTK_MESSAGE_ERROR,
+                                       GTK_BUTTONS_CLOSE,
+                                       _("Could not open the address \"%s\":\n%s"),
+                                       url, err->message);
+      
+      g_error_free (err);
+      
+      g_signal_connect (G_OBJECT (dialog),
+                        "response",
+                        G_CALLBACK (gtk_widget_destroy),
+                        NULL);
+      
+      gtk_widget_show (dialog);
+    }
+
+  g_free (url);
+  g_object_unref (G_OBJECT (screen));
+}
+
+static void
+open_url_callback (GtkWidget      *menu_item,
+                   TerminalScreen *screen)
+{
+  if (screen->priv->matched_string)
+    {
+      open_url (screen, screen->priv->matched_string);
+      g_free (screen->priv->matched_string);
+      screen->priv->matched_string = NULL;
+    }
+}
+
+static void
+copy_url_callback (GtkWidget      *menu_item,
+                   TerminalScreen *screen)
+{
+  if (screen->priv->matched_string)
+    {
+      gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE), 
+                              screen->priv->matched_string, -1);
+      g_free (screen->priv->matched_string);
+      screen->priv->matched_string = NULL;
+    }
+}
+
 
 static void
 popup_menu_detach (GtkWidget *attach_widget,
@@ -1084,6 +1255,27 @@ append_menuitem (GtkWidget  *menu,
   return menu_item;
 }
 
+static GtkWidget*
+append_stock_menuitem (GtkWidget  *menu,
+                       const char *text,
+                       GCallback   callback,
+                       gpointer    data)
+{
+  GtkWidget *menu_item;
+  
+  menu_item = gtk_image_menu_item_new_from_stock (text, NULL);
+  gtk_widget_show (menu_item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         menu_item);
+
+  if (callback)
+    g_signal_connect (G_OBJECT (menu_item),
+                      "activate",
+                      callback, data);
+
+  return menu_item;
+}
+
 static void
 terminal_screen_do_popup (TerminalScreen *screen,
                           GdkEventButton *event)
@@ -1100,21 +1292,44 @@ terminal_screen_do_popup (TerminalScreen *screen,
   g_assert (screen->priv->popup_menu == NULL);
   
   screen->priv->popup_menu = gtk_menu_new ();
+  gtk_menu_set_accel_group (GTK_MENU (screen->priv->popup_menu),
+                            terminal_accels_get_group_for_widget (screen->priv->popup_menu));
   
   gtk_menu_attach_to_widget (GTK_MENU (screen->priv->popup_menu),
                              GTK_WIDGET (screen->priv->zvt),
                              popup_menu_detach);
 
-  append_menuitem (screen->priv->popup_menu,
-                   _("_New window"),
-                   G_CALLBACK (new_window_callback),
-                   screen);
+  menu_item = append_menuitem (screen->priv->popup_menu,
+                               _("_New window"),
+                               G_CALLBACK (new_window_callback),
+                               screen);
+  gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
+                                ACCEL_PATH_NEW_WINDOW);
+  
+  menu_item = append_menuitem (screen->priv->popup_menu,
+                               _("New _tab"),
+                               G_CALLBACK (new_tab_callback),
+                               screen);
+  gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
+                                ACCEL_PATH_NEW_TAB);
 
-  append_menuitem (screen->priv->popup_menu,
-                   _("New _tab"),
-                   G_CALLBACK (new_tab_callback),
-                   screen);
+  menu_item = append_stock_menuitem (screen->priv->popup_menu,
+                                     GTK_STOCK_COPY,
+                                     G_CALLBACK (copy_callback),
+                                     screen);
+  gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
+                                ACCEL_PATH_COPY);
 
+  if (!terminal_screen_get_text_selected (screen))
+    gtk_widget_set_sensitive (menu_item, FALSE);
+
+  menu_item = append_stock_menuitem (screen->priv->popup_menu,
+                                     GTK_STOCK_PASTE,
+                                     G_CALLBACK (paste_callback),
+                                     screen);
+  gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
+                                ACCEL_PATH_PASTE);
+  
   profile_menu = gtk_menu_new ();
   menu_item = gtk_menu_item_new_with_mnemonic (_("_Profile"));
 
@@ -1181,11 +1396,26 @@ terminal_screen_do_popup (TerminalScreen *screen,
                      G_CALLBACK (show_menubar_callback),
                      screen);
 
-  
+#if 0
   append_menuitem (screen->priv->popup_menu,
                    _("Secure _Keyboard"),
                    G_CALLBACK (secure_keyboard_callback),
-                   screen);  
+                   screen);
+#endif
+
+  menu_item = append_menuitem (screen->priv->popup_menu,
+                               _("_Open Link"),
+                               G_CALLBACK (open_url_callback),
+                               screen);
+
+  gtk_widget_set_sensitive (menu_item, screen->priv->matched_string != NULL);
+  
+  menu_item = append_menuitem (screen->priv->popup_menu,
+                               _("_Copy Link Address"),
+                               G_CALLBACK (copy_url_callback),
+                               screen);
+
+  gtk_widget_set_sensitive (menu_item, screen->priv->matched_string != NULL);
   
   gtk_menu_popup (GTK_MENU (screen->priv->popup_menu),
                   NULL, NULL,
@@ -1202,13 +1432,46 @@ terminal_screen_popup_menu (GtkWidget      *zvt,
 }
 
 static gboolean
-terminal_screen_button_press_event (GtkWidget      *zvt,
+terminal_screen_button_press_event (GtkWidget      *widget,
                                     GdkEventButton *event,
                                     TerminalScreen *screen)
 {
+  ZvtTerm *term;
+  
+  term = ZVT_TERM (widget);
+
+  g_free (screen->priv->matched_string);
+  screen->priv->matched_string =
+    g_strdup (zvt_term_match_check (term,
+                                    event->x / term->charwidth,
+                                    event->y / term->charheight,
+                                    0));
+  
   if (event->button == 1 || event->button == 2)
     {
-      gtk_widget_grab_focus (zvt);
+      gtk_widget_grab_focus (widget);
+      
+      if (screen->priv->matched_string &&
+          (event->state & GDK_CONTROL_MASK))
+        {
+          open_url (screen, screen->priv->matched_string);
+          g_free (screen->priv->matched_string);
+          screen->priv->matched_string = NULL;
+          return TRUE; /* don't do anything else such as select with the click */
+        }
+      
+#if 0
+      /* old gnome-terminal had this code, but I'm not sure if
+       * it should be here. We always return FALSE, but
+       * sometimes old gnome-terminal didn't, so maybe that's
+       * why it had this.
+       */
+      if (event->button != 3
+          || (!(event->state & GDK_CONTROL_MASK) && term->vx->selected)
+          || (term->vx->vt.mode & VTMODE_SEND_MOUSE))
+        return FALSE;
+#endif      
+
       return FALSE; /* pass thru the click */
     }
   else if (event->button == 3)
